@@ -1,35 +1,76 @@
 """Transform class definitions from the mypy AST form to IR."""
 
+from __future__ import annotations
+
+import typing_extensions
 from abc import abstractmethod
-from typing import Callable, List, Optional, Set, Tuple
-from typing_extensions import Final
+from typing import Callable, Final
 
 from mypy.nodes import (
-    ClassDef, FuncDef, OverloadedFuncDef, PassStmt, AssignmentStmt, CallExpr, NameExpr, StrExpr,
-    ExpressionStmt, TempNode, Decorator, Lvalue, MemberExpr, RefExpr, TypeInfo, is_class_var
+    AssignmentStmt,
+    CallExpr,
+    ClassDef,
+    Decorator,
+    ExpressionStmt,
+    FuncDef,
+    Lvalue,
+    MemberExpr,
+    NameExpr,
+    OverloadedFuncDef,
+    PassStmt,
+    RefExpr,
+    StrExpr,
+    TempNode,
+    TypeInfo,
+    is_class_var,
 )
-from mypy.types import Instance, get_proper_type, ENUM_REMOVED_PROPS
+from mypy.types import ENUM_REMOVED_PROPS, Instance, UnboundType, get_proper_type
+from mypyc.common import PROPSET_PREFIX
+from mypyc.ir.class_ir import ClassIR, NonExtClassInfo
+from mypyc.ir.func_ir import FuncDecl, FuncSignature
 from mypyc.ir.ops import (
-    Value, Register, Call, LoadErrorValue, LoadStatic, InitStatic, TupleSet, SetAttr, Return,
-    BasicBlock, Branch, MethodCall, NAMESPACE_TYPE, LoadAddress
+    NAMESPACE_TYPE,
+    BasicBlock,
+    Branch,
+    Call,
+    InitStatic,
+    LoadAddress,
+    LoadErrorValue,
+    LoadStatic,
+    MethodCall,
+    Register,
+    Return,
+    SetAttr,
+    TupleSet,
+    Value,
 )
 from mypyc.ir.rtypes import (
-    RType, object_rprimitive, bool_rprimitive, dict_rprimitive, is_optional_type,
-    is_object_rprimitive, is_none_rprimitive
-)
-from mypyc.ir.func_ir import FuncDecl, FuncSignature
-from mypyc.ir.class_ir import ClassIR, NonExtClassInfo
-from mypyc.primitives.generic_ops import py_setattr_op, py_hasattr_op
-from mypyc.primitives.misc_ops import (
-    dataclass_sleight_of_hand, pytype_from_template_op, py_calc_meta_op, type_object_op,
-    not_implemented_op
-)
-from mypyc.primitives.dict_ops import dict_set_item_op, dict_new_op
-from mypyc.irbuild.util import (
-    is_dataclass_decorator, get_func_def, is_constant, dataclass_type
+    RType,
+    bool_rprimitive,
+    dict_rprimitive,
+    is_none_rprimitive,
+    is_object_rprimitive,
+    is_optional_type,
+    object_rprimitive,
 )
 from mypyc.irbuild.builder import IRBuilder
-from mypyc.irbuild.function import handle_ext_method, handle_non_ext_method, load_type
+from mypyc.irbuild.function import (
+    gen_property_getter_ir,
+    gen_property_setter_ir,
+    handle_ext_method,
+    handle_non_ext_method,
+    load_type,
+)
+from mypyc.irbuild.util import dataclass_type, get_func_def, is_constant, is_dataclass_decorator
+from mypyc.primitives.dict_ops import dict_new_op, dict_set_item_op
+from mypyc.primitives.generic_ops import py_hasattr_op, py_setattr_op
+from mypyc.primitives.misc_ops import (
+    dataclass_sleight_of_hand,
+    not_implemented_op,
+    py_calc_meta_op,
+    pytype_from_template_op,
+    type_object_op,
+)
 
 
 def transform_class_def(builder: IRBuilder, cdef: ClassDef) -> None:
@@ -49,15 +90,18 @@ def transform_class_def(builder: IRBuilder, cdef: ClassDef) -> None:
     # We do this check here because the base field of parent
     # classes aren't necessarily populated yet at
     # prepare_class_def time.
-    if any(ir.base_mro[i].base != ir. base_mro[i + 1] for i in range(len(ir.base_mro) - 1)):
-        builder.error("Non-trait MRO must be linear", cdef.line)
+    if any(ir.base_mro[i].base != ir.base_mro[i + 1] for i in range(len(ir.base_mro) - 1)):
+        builder.error("Multiple inheritance is not supported (except for traits)", cdef.line)
 
     if ir.allow_interpreted_subclasses:
         for parent in ir.mro:
             if not parent.allow_interpreted_subclasses:
                 builder.error(
                     'Base class "{}" does not allow interpreted subclasses'.format(
-                        parent.fullname), cdef.line)
+                        parent.fullname
+                    ),
+                    cdef.line,
+                )
 
     # Currently, we only create non-extension classes for classes that are
     # decorated or inherit from Enum. Classes decorated with @trait do not
@@ -66,9 +110,9 @@ def transform_class_def(builder: IRBuilder, cdef: ClassDef) -> None:
         cls_type = dataclass_type(cdef)
         if cls_type is None:
             cls_builder: ClassBuilder = ExtClassBuilder(builder, cdef)
-        elif cls_type in ['dataclasses', 'attr-auto']:
+        elif cls_type in ["dataclasses", "attr-auto"]:
             cls_builder = DataClassBuilder(builder, cdef)
-        elif cls_type == 'attr':
+        elif cls_type == "attr":
             cls_builder = AttrsClassBuilder(builder, cdef)
         else:
             raise ValueError(cls_type)
@@ -80,8 +124,7 @@ def transform_class_def(builder: IRBuilder, cdef: ClassDef) -> None:
             if isinstance(cls_builder, NonExtClassBuilder):
                 # properties with both getters and setters in non_extension
                 # classes not supported
-                builder.error("Property setters not supported in non-extension classes",
-                              stmt.line)
+                builder.error("Property setters not supported in non-extension classes", stmt.line)
             for item in stmt.items:
                 with builder.catch_errors(stmt.line):
                     cls_builder.add_method(get_func_def(item))
@@ -101,8 +144,9 @@ def transform_class_def(builder: IRBuilder, cdef: ClassDef) -> None:
                 continue
             lvalue = stmt.lvalues[0]
             if not isinstance(lvalue, NameExpr):
-                builder.error("Only assignment to variables is supported in class bodies",
-                              stmt.line)
+                builder.error(
+                    "Only assignment to variables is supported in class bodies", stmt.line
+                )
                 continue
             # We want to collect class variables in a dictionary for both real
             # non-extension classes and fake dataclass ones.
@@ -114,6 +158,26 @@ def transform_class_def(builder: IRBuilder, cdef: ClassDef) -> None:
         else:
             builder.error("Unsupported statement in class body", stmt.line)
 
+    # Generate implicit property setters/getters
+    for name, decl in ir.method_decls.items():
+        if decl.implicit and decl.is_prop_getter:
+            getter_ir = gen_property_getter_ir(builder, decl, cdef, ir.is_trait)
+            builder.functions.append(getter_ir)
+            ir.methods[getter_ir.decl.name] = getter_ir
+
+            setter_ir = None
+            setter_name = PROPSET_PREFIX + name
+            if setter_name in ir.method_decls:
+                setter_ir = gen_property_setter_ir(
+                    builder, ir.method_decls[setter_name], cdef, ir.is_trait
+                )
+                builder.functions.append(setter_ir)
+                ir.methods[setter_name] = setter_ir
+
+            ir.properties[name] = (getter_ir, setter_ir)
+            # TODO: Generate glue method if needed?
+            # TODO: Do we need interpreted glue methods? Maybe not?
+
     cls_builder.finalize(ir)
 
 
@@ -126,7 +190,7 @@ class ClassBuilder:
     def __init__(self, builder: IRBuilder, cdef: ClassDef) -> None:
         self.builder = builder
         self.cdef = cdef
-        self.attrs_to_cache: List[Tuple[Lvalue, RType]] = []
+        self.attrs_to_cache: list[tuple[Lvalue, RType]] = []
 
     @abstractmethod
     def add_method(self, fdef: FuncDef) -> None:
@@ -149,8 +213,9 @@ class NonExtClassBuilder(ClassBuilder):
     def create_non_ext_info(self) -> NonExtClassInfo:
         non_ext_bases = populate_non_ext_bases(self.builder, self.cdef)
         non_ext_metaclass = find_non_ext_metaclass(self.builder, self.cdef, non_ext_bases)
-        non_ext_dict = setup_non_ext_dict(self.builder, self.cdef, non_ext_metaclass,
-                                          non_ext_bases)
+        non_ext_dict = setup_non_ext_dict(
+            self.builder, self.cdef, non_ext_metaclass, non_ext_bases
+        )
         # We populate __annotations__ for non-extension classes
         # because dataclasses uses it to determine which attributes to compute on.
         # TODO: Maybe generate more precise types for annotations
@@ -162,8 +227,9 @@ class NonExtClassBuilder(ClassBuilder):
 
     def add_attr(self, lvalue: NameExpr, stmt: AssignmentStmt) -> None:
         add_non_ext_class_attr_ann(self.builder, self.non_ext, lvalue, stmt)
-        add_non_ext_class_attr(self.builder, self.non_ext, lvalue, stmt, self.cdef,
-                               self.attrs_to_cache)
+        add_non_ext_class_attr(
+            self.builder, self.non_ext, lvalue, stmt, self.cdef, self.attrs_to_cache
+        )
 
     def finalize(self, ir: ClassIR) -> None:
         # Dynamically create the class via the type constructor
@@ -171,16 +237,20 @@ class NonExtClassBuilder(ClassBuilder):
         non_ext_class = load_decorated_class(self.builder, self.cdef, non_ext_class)
 
         # Save the decorated class
-        self.builder.add(InitStatic(non_ext_class, self.cdef.name, self.builder.module_name,
-                                    NAMESPACE_TYPE))
+        self.builder.add(
+            InitStatic(non_ext_class, self.cdef.name, self.builder.module_name, NAMESPACE_TYPE)
+        )
 
         # Add the non-extension class to the dict
-        self.builder.call_c(dict_set_item_op,
-                            [
-                                self.builder.load_globals_dict(),
-                                self.builder.load_str(self.cdef.name),
-                                non_ext_class
-                            ], self.cdef.line)
+        self.builder.call_c(
+            dict_set_item_op,
+            [
+                self.builder.load_globals_dict(),
+                self.builder.load_str(self.cdef.name),
+                non_ext_class,
+            ],
+            self.cdef.line,
+        )
 
         # Cache any cacheable class attributes
         cache_class_attrs(self.builder, self.attrs_to_cache, self.cdef)
@@ -190,7 +260,7 @@ class ExtClassBuilder(ClassBuilder):
     def __init__(self, builder: IRBuilder, cdef: ClassDef) -> None:
         super().__init__(builder, cdef)
         # If the class is not decorated, generate an extension class for it.
-        self.type_obj: Optional[Value] = allocate_class(builder, cdef)
+        self.type_obj: Value | None = allocate_class(builder, cdef)
 
     def skip_attr_default(self, name: str, stmt: AssignmentStmt) -> bool:
         """Controls whether to skip generating a default for an attribute."""
@@ -209,13 +279,15 @@ class ExtClassBuilder(ClassBuilder):
         typ = self.builder.load_native_type_object(self.cdef.fullname)
         value = self.builder.accept(stmt.rvalue)
         self.builder.call_c(
-            py_setattr_op, [typ, self.builder.load_str(lvalue.name), value], stmt.line)
+            py_setattr_op, [typ, self.builder.load_str(lvalue.name), value], stmt.line
+        )
         if self.builder.non_function_scope() and stmt.is_final_def:
             self.builder.init_final_static(lvalue, value, self.cdef.name)
 
     def finalize(self, ir: ClassIR) -> None:
         attrs_with_defaults, default_assignments = find_attr_initializers(
-            self.builder, self.cdef, self.skip_attr_default)
+            self.builder, self.cdef, self.skip_attr_default
+        )
         ir.attrs_with_defaults.update(attrs_with_defaults)
         generate_attr_defaults_init(self.builder, self.cdef, default_assignments)
         create_ne_from_eq(self.builder, self.cdef)
@@ -242,13 +314,13 @@ class DataClassBuilder(ExtClassBuilder):
             self.builder.call_c(dict_new_op, [], self.cdef.line),
             self.builder.add(TupleSet([], self.cdef.line)),
             self.builder.call_c(dict_new_op, [], self.cdef.line),
-            self.builder.add(LoadAddress(type_object_op.type, type_object_op.src, self.cdef.line))
+            self.builder.add(LoadAddress(type_object_op.type, type_object_op.src, self.cdef.line)),
         )
 
     def skip_attr_default(self, name: str, stmt: AssignmentStmt) -> bool:
         return stmt.type is not None
 
-    def get_type_annotation(self, stmt: AssignmentStmt) -> Optional[TypeInfo]:
+    def get_type_annotation(self, stmt: AssignmentStmt) -> TypeInfo | None:
         # We populate __annotations__ because dataclasses uses it to determine
         # which attributes to compute on.
         ann_type = get_proper_type(stmt.type)
@@ -257,10 +329,12 @@ class DataClassBuilder(ExtClassBuilder):
         return None
 
     def add_attr(self, lvalue: NameExpr, stmt: AssignmentStmt) -> None:
-        add_non_ext_class_attr_ann(self.builder, self.non_ext, lvalue, stmt,
-                                   self.get_type_annotation)
-        add_non_ext_class_attr(self.builder, self.non_ext, lvalue, stmt, self.cdef,
-                               self.attrs_to_cache)
+        add_non_ext_class_attr_ann(
+            self.builder, self.non_ext, lvalue, stmt, self.get_type_annotation
+        )
+        add_non_ext_class_attr(
+            self.builder, self.non_ext, lvalue, stmt, self.cdef, self.attrs_to_cache
+        )
         super().add_attr(lvalue, stmt)
 
     def finalize(self, ir: ClassIR) -> None:
@@ -283,13 +357,17 @@ class DataClassBuilder(ExtClassBuilder):
         """
         super().finalize(ir)
         assert self.type_obj
-        add_dunders_to_non_ext_dict(self.builder, self.non_ext, self.cdef.line,
-                                    self.add_annotations_to_dict)
+        add_dunders_to_non_ext_dict(
+            self.builder, self.non_ext, self.cdef.line, self.add_annotations_to_dict
+        )
         dec = self.builder.accept(
-            next(d for d in self.cdef.decorators if is_dataclass_decorator(d)))
+            next(d for d in self.cdef.decorators if is_dataclass_decorator(d))
+        )
         self.builder.call_c(
-            dataclass_sleight_of_hand, [dec, self.type_obj, self.non_ext.dict, self.non_ext.anns],
-            self.cdef.line)
+            dataclass_sleight_of_hand,
+            [dec, self.type_obj, self.non_ext.dict, self.non_ext.anns],
+            self.cdef.line,
+        )
 
 
 class AttrsClassBuilder(DataClassBuilder):
@@ -306,14 +384,16 @@ class AttrsClassBuilder(DataClassBuilder):
     def skip_attr_default(self, name: str, stmt: AssignmentStmt) -> bool:
         return True
 
-    def get_type_annotation(self, stmt: AssignmentStmt) -> Optional[TypeInfo]:
+    def get_type_annotation(self, stmt: AssignmentStmt) -> TypeInfo | None:
         if isinstance(stmt.rvalue, CallExpr):
             # find the type arg in `attr.ib(type=str)`
             callee = stmt.rvalue.callee
-            if (isinstance(callee, MemberExpr) and
-                    callee.fullname in ['attr.ib', 'attr.attr'] and
-                    'type' in stmt.rvalue.arg_names):
-                index = stmt.rvalue.arg_names.index('type')
+            if (
+                isinstance(callee, MemberExpr)
+                and callee.fullname in ["attr.ib", "attr.attr"]
+                and "type" in stmt.rvalue.arg_names
+            ):
+                index = stmt.rvalue.arg_names.index("type")
                 type_name = stmt.rvalue.args[index]
                 if isinstance(type_name, NameExpr) and isinstance(type_name.node, TypeInfo):
                     lvalue = stmt.lvalues[0]
@@ -331,42 +411,53 @@ def allocate_class(builder: IRBuilder, cdef: ClassDef) -> Value:
     else:
         tp_bases = builder.add(LoadErrorValue(object_rprimitive, is_borrowed=True))
     modname = builder.load_str(builder.module_name)
-    template = builder.add(LoadStatic(object_rprimitive, cdef.name + "_template",
-                                      builder.module_name, NAMESPACE_TYPE))
+    template = builder.add(
+        LoadStatic(object_rprimitive, cdef.name + "_template", builder.module_name, NAMESPACE_TYPE)
+    )
     # Create the class
-    tp = builder.call_c(pytype_from_template_op,
-                        [template, tp_bases, modname], cdef.line)
+    tp = builder.call_c(pytype_from_template_op, [template, tp_bases, modname], cdef.line)
     # Immediately fix up the trait vtables, before doing anything with the class.
     ir = builder.mapper.type_to_ir[cdef.info]
     if not ir.is_trait and not ir.builtin_base:
-        builder.add(Call(
-            FuncDecl(cdef.name + '_trait_vtable_setup',
-                     None, builder.module_name,
-                     FuncSignature([], bool_rprimitive)), [], -1))
+        builder.add(
+            Call(
+                FuncDecl(
+                    cdef.name + "_trait_vtable_setup",
+                    None,
+                    builder.module_name,
+                    FuncSignature([], bool_rprimitive),
+                ),
+                [],
+                -1,
+            )
+        )
     # Populate a '__mypyc_attrs__' field containing the list of attrs
-    builder.call_c(py_setattr_op, [
-        tp, builder.load_str('__mypyc_attrs__'),
-        create_mypyc_attrs_tuple(builder, builder.mapper.type_to_ir[cdef.info], cdef.line)],
-        cdef.line)
+    builder.call_c(
+        py_setattr_op,
+        [
+            tp,
+            builder.load_str("__mypyc_attrs__"),
+            create_mypyc_attrs_tuple(builder, builder.mapper.type_to_ir[cdef.info], cdef.line),
+        ],
+        cdef.line,
+    )
 
     # Save the class
     builder.add(InitStatic(tp, cdef.name, builder.module_name, NAMESPACE_TYPE))
 
     # Add it to the dict
-    builder.call_c(dict_set_item_op,
-                   [builder.load_globals_dict(),
-                    builder.load_str(cdef.name),
-                    tp],
-                   cdef.line)
+    builder.call_c(
+        dict_set_item_op, [builder.load_globals_dict(), builder.load_str(cdef.name), tp], cdef.line
+    )
 
     return tp
 
 
 # Mypy uses these internally as base classes of TypedDict classes. These are
 # lies and don't have any runtime equivalent.
-MAGIC_TYPED_DICT_CLASSES: Final[Tuple[str, ...]] = (
-    'typing._TypedDict',
-    'typing_extensions._TypedDict',
+MAGIC_TYPED_DICT_CLASSES: Final[tuple[str, ...]] = (
+    "typing._TypedDict",
+    "typing_extensions._TypedDict",
 )
 
 
@@ -379,13 +470,16 @@ def populate_non_ext_bases(builder: IRBuilder, cdef: ClassDef) -> Value:
     ir = builder.mapper.type_to_ir[cdef.info]
     bases = []
     for cls in cdef.info.mro[1:]:
-        if cls.fullname == 'builtins.object':
+        if cls.fullname == "builtins.object":
             continue
-        if is_named_tuple and cls.fullname in ('typing.Sequence',
-                                               'typing.Iterable',
-                                               'typing.Collection',
-                                               'typing.Reversible',
-                                               'typing.Container'):
+        if is_named_tuple and cls.fullname in (
+            "typing.Sequence",
+            "typing.Iterable",
+            "typing.Collection",
+            "typing.Reversible",
+            "typing.Container",
+            "typing.Sized",
+        ):
             # HAX: Synthesized base classes added by mypy don't exist at runtime, so skip them.
             #      This could break if they were added explicitly, though...
             continue
@@ -398,25 +492,37 @@ def populate_non_ext_bases(builder: IRBuilder, cdef: ClassDef) -> Value:
         if cls.fullname in MAGIC_TYPED_DICT_CLASSES:
             # HAX: Mypy internally represents TypedDict classes differently from what
             #      should happen at runtime. Replace with something that works.
-            module = 'typing'
+            module = "typing"
             if builder.options.capi_version < (3, 9):
-                name = 'TypedDict'
+                name = "TypedDict"
                 if builder.options.capi_version < (3, 8):
                     # TypedDict was added to typing in Python 3.8.
-                    module = 'typing_extensions'
+                    module = "typing_extensions"
+                    # TypedDict is not a real type on typing_extensions 4.7.0+
+                    name = "_TypedDict"
+                    if isinstance(typing_extensions.TypedDict, type):
+                        raise RuntimeError(
+                            "It looks like you may have an old version "
+                            "of typing_extensions installed. "
+                            "typing_extensions>=4.7.0 is required on Python 3.7."
+                        )
             else:
                 # In Python 3.9 TypedDict is not a real type.
-                name = '_TypedDict'
+                name = "_TypedDict"
             base = builder.get_module_attr(module, name, cdef.line)
-        elif is_named_tuple and cls.fullname == 'builtins.tuple':
+        elif is_named_tuple and cls.fullname == "builtins.tuple":
             if builder.options.capi_version < (3, 9):
-                name = 'NamedTuple'
+                name = "NamedTuple"
             else:
                 # This was changed in Python 3.9.
-                name = '_NamedTuple'
-            base = builder.get_module_attr('typing', name, cdef.line)
+                name = "_NamedTuple"
+            base = builder.get_module_attr("typing", name, cdef.line)
         else:
-            base = builder.load_global_str(cls.name, cdef.line)
+            cls_module = cls.fullname.rsplit(".", 1)[0]
+            if cls_module == builder.current_module:
+                base = builder.load_global_str(cls.name, cdef.line)
+            else:
+                base = builder.load_module_attr_by_fullname(cls.fullname, cdef.line)
         bases.append(base)
         if cls.fullname in MAGIC_TYPED_DICT_CLASSES:
             # The remaining base classes are synthesized by mypy and should be ignored.
@@ -425,46 +531,46 @@ def populate_non_ext_bases(builder: IRBuilder, cdef: ClassDef) -> Value:
 
 
 def find_non_ext_metaclass(builder: IRBuilder, cdef: ClassDef, bases: Value) -> Value:
-    """Find the metaclass of a class from its defs and bases. """
+    """Find the metaclass of a class from its defs and bases."""
     if cdef.metaclass:
         declared_metaclass = builder.accept(cdef.metaclass)
     else:
         if cdef.info.typeddict_type is not None and builder.options.capi_version >= (3, 9):
             # In Python 3.9, the metaclass for class-based TypedDict is typing._TypedDictMeta.
             # We can't easily calculate it generically, so special case it.
-            return builder.get_module_attr('typing', '_TypedDictMeta', cdef.line)
+            return builder.get_module_attr("typing", "_TypedDictMeta", cdef.line)
         elif cdef.info.is_named_tuple and builder.options.capi_version >= (3, 9):
             # In Python 3.9, the metaclass for class-based NamedTuple is typing.NamedTupleMeta.
             # We can't easily calculate it generically, so special case it.
-            return builder.get_module_attr('typing', 'NamedTupleMeta', cdef.line)
+            return builder.get_module_attr("typing", "NamedTupleMeta", cdef.line)
 
-        declared_metaclass = builder.add(LoadAddress(type_object_op.type,
-                                                     type_object_op.src, cdef.line))
+        declared_metaclass = builder.add(
+            LoadAddress(type_object_op.type, type_object_op.src, cdef.line)
+        )
 
     return builder.call_c(py_calc_meta_op, [declared_metaclass, bases], cdef.line)
 
 
-def setup_non_ext_dict(builder: IRBuilder,
-                       cdef: ClassDef,
-                       metaclass: Value,
-                       bases: Value) -> Value:
+def setup_non_ext_dict(
+    builder: IRBuilder, cdef: ClassDef, metaclass: Value, bases: Value
+) -> Value:
     """Initialize the class dictionary for a non-extension class.
 
     This class dictionary is passed to the metaclass constructor.
     """
     # Check if the metaclass defines a __prepare__ method, and if so, call it.
-    has_prepare = builder.call_c(py_hasattr_op,
-                                 [metaclass,
-                                  builder.load_str('__prepare__')], cdef.line)
+    has_prepare = builder.call_c(
+        py_hasattr_op, [metaclass, builder.load_str("__prepare__")], cdef.line
+    )
 
     non_ext_dict = Register(dict_rprimitive)
 
-    true_block, false_block, exit_block, = BasicBlock(), BasicBlock(), BasicBlock()
+    true_block, false_block, exit_block = BasicBlock(), BasicBlock(), BasicBlock()
     builder.add_bool_branch(has_prepare, true_block, false_block)
 
     builder.activate_block(true_block)
     cls_name = builder.load_str(cdef.name)
-    prepare_meth = builder.py_get_attr(metaclass, '__prepare__', cdef.line)
+    prepare_meth = builder.py_get_attr(metaclass, "__prepare__", cdef.line)
     prepare_dict = builder.py_call(prepare_meth, [cls_name, bases], cdef.line)
     builder.assign(non_ext_dict, prepare_dict, cdef.line)
     builder.goto(exit_block)
@@ -477,15 +583,16 @@ def setup_non_ext_dict(builder: IRBuilder,
     return non_ext_dict
 
 
-def add_non_ext_class_attr_ann(builder: IRBuilder,
-                               non_ext: NonExtClassInfo,
-                               lvalue: NameExpr,
-                               stmt: AssignmentStmt,
-                               get_type_info: Optional[Callable[[AssignmentStmt],
-                                                                Optional[TypeInfo]]] = None
-                               ) -> None:
+def add_non_ext_class_attr_ann(
+    builder: IRBuilder,
+    non_ext: NonExtClassInfo,
+    lvalue: NameExpr,
+    stmt: AssignmentStmt,
+    get_type_info: Callable[[AssignmentStmt], TypeInfo | None] | None = None,
+) -> None:
     """Add a class attribute to __annotations__ of a non-extension class."""
-    typ: Optional[Value] = None
+    # FIXME: try to better preserve the special forms and type parameters of generics.
+    typ: Value | None = None
     if get_type_info is not None:
         type_info = get_type_info(stmt)
         if type_info:
@@ -494,7 +601,17 @@ def add_non_ext_class_attr_ann(builder: IRBuilder,
     if typ is None:
         # FIXME: if get_type_info is not provided, don't fall back to stmt.type?
         ann_type = get_proper_type(stmt.type)
-        if isinstance(ann_type, Instance):
+        if (
+            isinstance(stmt.unanalyzed_type, UnboundType)
+            and stmt.unanalyzed_type.original_str_expr is not None
+        ):
+            # Annotation is a forward reference, so don't attempt to load the actual
+            # type and load the string instead.
+            #
+            # TODO: is it possible to determine whether a non-string annotation is
+            # actually a forward reference due to the __annotations__ future?
+            typ = builder.load_str(stmt.unanalyzed_type.original_str_expr)
+        elif isinstance(ann_type, Instance):
             typ = load_type(builder, ann_type.type, stmt.line)
         else:
             typ = builder.add(LoadAddress(type_object_op.type, type_object_op.src, stmt.line))
@@ -503,12 +620,14 @@ def add_non_ext_class_attr_ann(builder: IRBuilder,
     builder.call_c(dict_set_item_op, [non_ext.anns, key, typ], stmt.line)
 
 
-def add_non_ext_class_attr(builder: IRBuilder,
-                           non_ext: NonExtClassInfo,
-                           lvalue: NameExpr,
-                           stmt: AssignmentStmt,
-                           cdef: ClassDef,
-                           attr_to_cache: List[Tuple[Lvalue, RType]]) -> None:
+def add_non_ext_class_attr(
+    builder: IRBuilder,
+    non_ext: NonExtClassInfo,
+    lvalue: NameExpr,
+    stmt: AssignmentStmt,
+    cdef: ClassDef,
+    attr_to_cache: list[tuple[Lvalue, RType]],
+) -> None:
     """Add a class attribute to __dict__ of a non-extension class."""
     # Only add the attribute to the __dict__ if the assignment is of the form:
     # x: type = value (don't add attributes of the form 'x: type' to the __dict__).
@@ -519,7 +638,7 @@ def add_non_ext_class_attr(builder: IRBuilder,
         # are final.
         if (
             cdef.info.bases
-            and cdef.info.bases[0].type.fullname == 'enum.Enum'
+            and cdef.info.bases[0].type.fullname == "enum.Enum"
             # Skip these since Enum will remove it
             and lvalue.name not in ENUM_REMOVED_PROPS
         ):
@@ -527,10 +646,9 @@ def add_non_ext_class_attr(builder: IRBuilder,
             attr_to_cache.append((lvalue, object_rprimitive))
 
 
-def find_attr_initializers(builder: IRBuilder,
-                           cdef: ClassDef,
-                           skip: Optional[Callable[[str, AssignmentStmt], bool]] = None,
-                           ) -> Tuple[Set[str], List[AssignmentStmt]]:
+def find_attr_initializers(
+    builder: IRBuilder, cdef: ClassDef, skip: Callable[[str, AssignmentStmt], bool] | None = None
+) -> tuple[set[str], list[AssignmentStmt]]:
     """Find initializers of attributes in a class body.
 
     If provided, the skip arg should be a callable which will return whether
@@ -550,15 +668,17 @@ def find_attr_initializers(builder: IRBuilder,
         if info not in builder.mapper.type_to_ir:
             continue
         for stmt in info.defn.defs.body:
-            if (isinstance(stmt, AssignmentStmt)
-                    and isinstance(stmt.lvalues[0], NameExpr)
-                    and not is_class_var(stmt.lvalues[0])
-                    and not isinstance(stmt.rvalue, TempNode)):
+            if (
+                isinstance(stmt, AssignmentStmt)
+                and isinstance(stmt.lvalues[0], NameExpr)
+                and not is_class_var(stmt.lvalues[0])
+                and not isinstance(stmt.rvalue, TempNode)
+            ):
                 name = stmt.lvalues[0].name
-                if name == '__slots__':
+                if name == "__slots__":
                     continue
 
-                if name == '__deletable__':
+                if name == "__deletable__":
                     check_deletable_declaration(builder, cls, stmt.line)
                     continue
 
@@ -569,9 +689,12 @@ def find_attr_initializers(builder: IRBuilder,
 
                 # If the attribute is initialized to None and type isn't optional,
                 # doesn't initialize it to anything (special case for "# type:" comments).
-                if isinstance(stmt.rvalue, RefExpr) and stmt.rvalue.fullname == 'builtins.None':
-                    if (not is_optional_type(attr_type) and not is_object_rprimitive(attr_type)
-                            and not is_none_rprimitive(attr_type)):
+                if isinstance(stmt.rvalue, RefExpr) and stmt.rvalue.fullname == "builtins.None":
+                    if (
+                        not is_optional_type(attr_type)
+                        and not is_object_rprimitive(attr_type)
+                        and not is_none_rprimitive(attr_type)
+                    ):
                         continue
 
                 attrs_with_defaults.add(name)
@@ -580,9 +703,9 @@ def find_attr_initializers(builder: IRBuilder,
     return attrs_with_defaults, default_assignments
 
 
-def generate_attr_defaults_init(builder: IRBuilder,
-                                cdef: ClassDef,
-                                default_assignments: List[AssignmentStmt]) -> None:
+def generate_attr_defaults_init(
+    builder: IRBuilder, cdef: ClassDef, default_assignments: list[AssignmentStmt]
+) -> None:
     """Generate an initialization method for default attr values (from class vars)."""
     if not default_assignments:
         return
@@ -590,13 +713,13 @@ def generate_attr_defaults_init(builder: IRBuilder,
     if cls.builtin_base:
         return
 
-    with builder.enter_method(cls, '__mypyc_defaults_setup', bool_rprimitive):
+    with builder.enter_method(cls, "__mypyc_defaults_setup", bool_rprimitive):
         self_var = builder.self()
         for stmt in default_assignments:
             lvalue = stmt.lvalues[0]
             assert isinstance(lvalue, NameExpr)
             if not stmt.is_final_def and not is_constant(stmt.rvalue):
-                builder.warning('Unsupported default attribute value', stmt.rvalue.line)
+                builder.warning("Unsupported default attribute value", stmt.rvalue.line)
 
             attr_type = cls.attr_type(lvalue.name)
             val = builder.coerce(builder.accept(stmt.rvalue), attr_type, stmt.line)
@@ -619,55 +742,58 @@ def check_deletable_declaration(builder: IRBuilder, cl: ClassIR, line: int) -> N
                     break
             else:
                 _, base = cl.attr_details(attr)
-                builder.error(('Attribute "{}" not defined in "{}" ' +
-                               '(defined in "{}")').format(attr, cl.name, base.name), line)
+                builder.error(
+                    ('Attribute "{}" not defined in "{}" ' + '(defined in "{}")').format(
+                        attr, cl.name, base.name
+                    ),
+                    line,
+                )
 
 
 def create_ne_from_eq(builder: IRBuilder, cdef: ClassDef) -> None:
     """Create a "__ne__" method from a "__eq__" method (if only latter exists)."""
     cls = builder.mapper.type_to_ir[cdef.info]
-    if cls.has_method('__eq__') and not cls.has_method('__ne__'):
+    if cls.has_method("__eq__") and not cls.has_method("__ne__"):
         gen_glue_ne_method(builder, cls, cdef.line)
 
 
 def gen_glue_ne_method(builder: IRBuilder, cls: ClassIR, line: int) -> None:
-    """Generate a "__ne__" method from a "__eq__" method. """
-    with builder.enter_method(cls, '__ne__', object_rprimitive):
-        rhs_arg = builder.add_argument('rhs', object_rprimitive)
+    """Generate a "__ne__" method from a "__eq__" method."""
+    with builder.enter_method(cls, "__ne__", object_rprimitive):
+        rhs_arg = builder.add_argument("rhs", object_rprimitive)
 
         # If __eq__ returns NotImplemented, then __ne__ should also
         not_implemented_block, regular_block = BasicBlock(), BasicBlock()
-        eqval = builder.add(MethodCall(builder.self(), '__eq__', [rhs_arg], line))
-        not_implemented = builder.add(LoadAddress(not_implemented_op.type,
-                                                  not_implemented_op.src, line))
-        builder.add(Branch(
-            builder.translate_is_op(eqval, not_implemented, 'is', line),
-            not_implemented_block,
-            regular_block,
-            Branch.BOOL))
+        eqval = builder.add(MethodCall(builder.self(), "__eq__", [rhs_arg], line))
+        not_implemented = builder.add(
+            LoadAddress(not_implemented_op.type, not_implemented_op.src, line)
+        )
+        builder.add(
+            Branch(
+                builder.translate_is_op(eqval, not_implemented, "is", line),
+                not_implemented_block,
+                regular_block,
+                Branch.BOOL,
+            )
+        )
 
         builder.activate_block(regular_block)
-        retval = builder.coerce(
-            builder.unary_op(eqval, 'not', line), object_rprimitive, line
-        )
+        retval = builder.coerce(builder.unary_op(eqval, "not", line), object_rprimitive, line)
         builder.add(Return(retval))
 
         builder.activate_block(not_implemented_block)
         builder.add(Return(not_implemented))
 
 
-def load_non_ext_class(builder: IRBuilder,
-                       ir: ClassIR,
-                       non_ext: NonExtClassInfo,
-                       line: int) -> Value:
+def load_non_ext_class(
+    builder: IRBuilder, ir: ClassIR, non_ext: NonExtClassInfo, line: int
+) -> Value:
     cls_name = builder.load_str(ir.name)
 
     add_dunders_to_non_ext_dict(builder, non_ext, line)
 
     class_type_obj = builder.py_call(
-        non_ext.metaclass,
-        [cls_name, non_ext.bases, non_ext.dict],
-        line
+        non_ext.metaclass, [cls_name, non_ext.bases, non_ext.dict], line
     )
     return class_type_obj
 
@@ -690,9 +816,9 @@ def load_decorated_class(builder: IRBuilder, cdef: ClassDef, type_obj: Value) ->
     return dec_class
 
 
-def cache_class_attrs(builder: IRBuilder,
-                      attrs_to_cache: List[Tuple[Lvalue, RType]],
-                      cdef: ClassDef) -> None:
+def cache_class_attrs(
+    builder: IRBuilder, attrs_to_cache: list[tuple[Lvalue, RType]], cdef: ClassDef
+) -> None:
     """Add class attributes to be cached to the global cache."""
     typ = builder.load_native_type_object(cdef.info.fullname)
     for lval, rtype in attrs_to_cache:
@@ -704,22 +830,21 @@ def cache_class_attrs(builder: IRBuilder,
 def create_mypyc_attrs_tuple(builder: IRBuilder, ir: ClassIR, line: int) -> Value:
     attrs = [name for ancestor in ir.mro for name in ancestor.attributes]
     if ir.inherits_python:
-        attrs.append('__dict__')
+        attrs.append("__dict__")
     items = [builder.load_str(attr) for attr in attrs]
     return builder.new_tuple(items, line)
 
 
-def add_dunders_to_non_ext_dict(builder: IRBuilder, non_ext: NonExtClassInfo,
-                                line: int, add_annotations: bool = True) -> None:
+def add_dunders_to_non_ext_dict(
+    builder: IRBuilder, non_ext: NonExtClassInfo, line: int, add_annotations: bool = True
+) -> None:
     if add_annotations:
         # Add __annotations__ to the class dict.
-        builder.add_to_non_ext_dict(non_ext, '__annotations__', non_ext.anns, line)
+        builder.add_to_non_ext_dict(non_ext, "__annotations__", non_ext.anns, line)
 
     # We add a __doc__ attribute so if the non-extension class is decorated with the
     # dataclass decorator, dataclass will not try to look for __text_signature__.
     # https://github.com/python/cpython/blob/3.7/Lib/dataclasses.py#L957
-    filler_doc_str = 'mypyc filler docstring'
-    builder.add_to_non_ext_dict(
-        non_ext, '__doc__', builder.load_str(filler_doc_str), line)
-    builder.add_to_non_ext_dict(
-        non_ext, '__module__', builder.load_str(builder.module_name), line)
+    filler_doc_str = "mypyc filler docstring"
+    builder.add_to_non_ext_dict(non_ext, "__doc__", builder.load_str(filler_doc_str), line)
+    builder.add_to_non_ext_dict(non_ext, "__module__", builder.load_str(builder.module_name), line)
